@@ -1,12 +1,14 @@
+import sys
+
 import numpy as np
 import pandas as pd
 from scipy.stats import rankdata
 
 from scattertext.PValGetter import get_p_vals
 from scattertext.Scalers import percentile_min, percentile_alphabetical
+from scattertext.ScatterChartData import ScatterChartData
 from scattertext.TermDocMatrixFilter import filter_bigrams_by_pmis, \
 	filter_out_unigrams_that_only_occur_in_one_bigram
-from scattertext.termranking import AbsoluteFrequencyRanker
 from scattertext.termscoring import ScaledFScore
 from scattertext.termscoring.CornerScore import CornerScore
 
@@ -15,66 +17,41 @@ class NoWordMeetsTermFrequencyRequirementsError(Exception):
 	pass
 
 
-class ScatterChartData(object):
-	def __init__(self,
-	             minimum_term_frequency=3,
-	             minimum_not_category_term_frequency=0,
-	             jitter=None,
-	             seed=0,
-	             pmi_threshold_coefficient=3,
-	             max_terms=None,
-	             filter_unigrams=False,
-	             term_ranker=AbsoluteFrequencyRanker,
-	             use_non_text_features=False,
-	             term_significance=None,
-	             terms_to_include=None):
-		'''
-
-		Parameters
-		----------
-		term_doc_matrix : TermDocMatrix
-			The term doc matrix to use for the scatter chart.
-		minimum_term_frequency : int, optional
-			Minimum times an ngram has to be seen to be included. Default is 3.
-		minimum_not_category_term_frequency : int, optional
-		  If an n-gram does not occur in the category, minimum times it
-		   must been seen to be included. Default is 0.
-		jitter : float, optional
-			Maximum amount of noise to be added to points, 0.2 is a lot. Default is None to disable jitter.
-		seed : float, optional
-			Random seed. Default 0
-		pmi_threshold_coefficient : int
-			Filter out bigrams with a PMI of < 2 * pmi_threshold_coefficient. Default is 3
-		max_terms : int, optional
-			Maximum number of terms to include in visualization
-		filter_unigrams : bool, optional
-			If True, remove unigrams that are part of bigrams. Default is False.
-		term_ranker : TermRanker, optional
-			TermRanker class for determining term frequency ranks.
-		use_non_text_features : bool, default = False
-			Use non-BoW features (e.g., Empath) instead of text features
-		term_significance : TermSignificance instance or None
-			Way of getting significance scores.  If None, p values will not be added.
-		terms_to_include : set or None
-			Only annotate these terms in chart
-		'''
-		self.jitter = jitter
-		self.minimum_term_frequency = minimum_term_frequency
-		self.minimum_not_category_term_frequency = minimum_not_category_term_frequency
-		self.seed = seed
-		self.pmi_threshold_coefficient = pmi_threshold_coefficient
-		self.filter_unigrams = filter_unigrams
-		self.term_ranker = term_ranker
-		self.max_terms = max_terms
-		self.use_non_text_features = use_non_text_features
-		self.term_significance = term_significance
-		self.terms_to_include = terms_to_include
-		np.random.seed(seed)
-
-
 class CoordinatesNotRightException(Exception): pass
 
 
+class TermDocMatrixHasNoMetadataException(Exception): pass
+
+def check_topic_model_string_format(term_dict):
+	'''
+	Parameters
+	----------
+	term_dict: dict {metadataname: [term1, term2, ....], ...}
+
+	Returns
+	-------
+	None
+	'''
+	if type(term_dict) != dict:
+		raise TypeError("Argument for term_dict must be a dict, keyed on strings, and contain a list of strings.")
+	for k, v in term_dict.items():
+		if type(v) != list:
+			raise TypeError("Values in term dict must only be lists.")
+
+		if sys.version_info[0] == 2:
+			if type(k) != str and type(k) != unicode:
+				raise TypeError("Keys in term dict must be of type str or unicode.")
+			for e in v:
+				if type(k) != str and type(k) != unicode:
+					raise TypeError("Values in term lists must be str or unicode.")
+
+		if sys.version_info[0] == 3:
+			if type(k) != str:
+				raise TypeError("Keys in term dict must be of type str.")
+			for e in v:
+				if type(e) != str:
+					raise TypeError("Values in term lists must be str.")
+
 class ScatterChart:
 	def __init__(self,
 	             term_doc_matrix,
@@ -95,6 +72,27 @@ class ScatterChart:
 		self._rescale_x = None
 		self._rescale_y = None
 		self.used = False
+		self.metadata_term_lists = None
+
+	def inject_metadata_term_lists(self, term_dict):
+		'''
+		Inserts dictionary of meta data terms into object.
+
+		Parameters
+		----------
+		term_dict: dict {metadataname: [term1, term2, ....], ...}
+
+		Returns
+		-------
+		self: ScatterChart
+		'''
+		check_topic_model_string_format(term_dict)
+
+		if not self.term_doc_matrix.metadata_in_use():
+			raise TermDocMatrixHasNoMetadataException("No metadata is present in the term document matrix")
+
+		self.metadata_term_lists = term_dict
+		return self
 
 	def inject_coordinates(self,
 	                       x_coords,
@@ -152,7 +150,8 @@ class ScatterChart:
 	            title_case_names=False,
 	            not_categories=None,
 	            neutral_categories=None,
-	            extra_categories=None):
+	            extra_categories=None,
+	            background_scorer=None):
 		'''
 
 		Parameters
@@ -175,6 +174,8 @@ class ScatterChart:
 			List of categories to use as neutral.  Defaults [].
 		extra_categories : list, optional
 			List of categories to use as extra.  Defaults [].
+		background_scorer : CharacteristicScorer, optional
+			Used for bg scores
 
 		Returns
 		-------
@@ -217,11 +218,8 @@ class ScatterChart:
 			                    if c not in [category] + not_categories + neutral_categories]
 		all_categories = [category] + not_categories + neutral_categories + extra_categories
 
-		term_ranker = self.scatterchartdata.term_ranker(self.term_doc_matrix)
-		if self.scatterchartdata.use_non_text_features:
-			term_ranker.use_non_text_features()
+		df = self._get_term_category_frequencies()
 
-		df = term_ranker.get_ranks()
 		if self.x_coords is not None:
 			df['x'] = self.x_coords
 			df['y'] = self.y_coords
@@ -240,18 +238,16 @@ class ScatterChart:
 
 		if scores is None:
 			scores = self._get_default_scores(category, not_categories, df)
-		# np.array(self.term_doc_matrix.get_rudder_scores(category))
-		# df['category score'] = np.array(self.term_doc_matrix.get_rudder_scores(category))
 		category_column_name = category + ' freq'
 		df['category score'] = CornerScore.get_scores_for_category(
 			df[category_column_name],
-			df[[c+' freq' for c in not_categories]].sum(axis=1)
+			df[[c + ' freq' for c in not_categories]].sum(axis=1)
 		)
 		if self.scatterchartdata.term_significance is not None:
 			df['p'] = get_p_vals(df, category_column_name,
 			                     self.scatterchartdata.term_significance)
 		df['not category score'] = CornerScore.get_scores_for_category(
-			df[[c+' freq' for c in not_categories]].sum(axis=1),
+			df[[c + ' freq' for c in not_categories]].sum(axis=1),
 			df[category_column_name]
 		)
 		df['color_scores'] = scores
@@ -274,8 +270,6 @@ class ScatterChart:
 			df = self._limit_max_terms(category, df)
 		df = df.reset_index()
 
-
-
 		if self.x_coords is None:
 			self.x_coords, self.y_coords = self._get_coordinates_from_transform_and_jitter_frequencies \
 				(category, df, not_categories, transform)
@@ -288,7 +282,6 @@ class ScatterChart:
 		if extra_categories != []:
 			df['extra cat freq'] = df[[x + ' freq' for x in extra_categories]].sum(axis=1).fillna(0)
 
-
 		json_df = df[['x', 'y', 'ox', 'oy', 'term']]
 
 		if self.scatterchartdata.term_significance:
@@ -296,7 +289,10 @@ class ScatterChart:
 		self._add_term_freq_to_json_df(json_df, df, category)
 		json_df['s'] = percentile_min(df['color_scores'])
 		json_df['os'] = df['color_scores']
-		if not self.scatterchartdata.use_non_text_features:
+		if background_scorer:
+			bg_scores = background_scorer.get_scores(self.term_doc_matrix)
+			json_df['bg'] = bg_scores[1].loc[json_df.term].values
+		elif not self.scatterchartdata.use_non_text_features:
 			json_df['bg'] = self._get_corpus_characteristic_scores(json_df)
 
 		self._preform_axis_rescale(json_df, self._rescale_x, 'x')
@@ -327,9 +323,14 @@ class ScatterChart:
 		              'categories': self.term_doc_matrix.get_categories(),
 		              'neutral_category_internal_names': neutral_categories,
 		              'extra_category_internal_names': extra_categories}}
+		if self.metadata_term_lists is not None:
+			j['metalists'] = self.metadata_term_lists
 		j['data'] = json_df.sort_values(by=['x', 'y', 'term']).to_dict(orient='records')
 		return j
 
+	def _get_term_category_frequencies(self):
+		return self.term_doc_matrix.get_term_category_frequencies(self.scatterchartdata)
+
 	def _use_only_selected_terms(self, json_df):
 		term_df = pd.DataFrame({"term": self.scatterchartdata.terms_to_include})
 		return pd.merge(json_df, term_df, on='term', how='inner')
@@ -351,14 +352,14 @@ class ScatterChart:
 	def _add_term_freq_to_json_df(self, json_df, term_freq_df, category):
 		json_df['cat25k'] = (((term_freq_df[category + ' freq'] * 1.
 		                       / term_freq_df[category + ' freq'].sum()) * 25000)
-			.apply(np.round).astype(np.int))
+		                     .apply(np.round).astype(np.int))
 		json_df['ncat25k'] = (((term_freq_df['not cat freq'] * 1.
 		                        / term_freq_df['not cat freq'].sum()) * 25000)
-			.apply(np.round).astype(np.int))
+		                      .apply(np.round).astype(np.int))
 		if 'neut cat freq' in term_freq_df:
 			json_df['neut25k'] = (((term_freq_df['neut cat freq'] * 1.
 			                        / term_freq_df['neut cat freq'].sum()) * 25000)
-				.apply(np.round).astype(np.int))
+			                      .apply(np.round).astype(np.int))
 			json_df['neut'] = term_freq_df['neut cat freq']
 		else:
 			json_df['neut25k'] = 0
@@ -366,15 +367,15 @@ class ScatterChart:
 		if 'extra cat freq' in term_freq_df:
 			json_df['extra25k'] = (((term_freq_df['extra cat freq'] * 1.
 			                         / term_freq_df['extra cat freq'].sum()) * 25000)
-				.apply(np.round).astype(np.int))
+			                       .apply(np.round).astype(np.int))
 			json_df['extra'] = term_freq_df['extra cat freq']
 		else:
 			json_df['extra25k'] = 0
 			json_df['extra'] = 0
 
 	def _get_category_names(self, category):
-		other_categories = [val + ' freq' for _, val \
-		                    in self.term_doc_matrix._category_idx_store.items() \
+		other_categories = [val + ' freq' for val \
+		                    in self.term_doc_matrix.get_categories() \
 		                    if val != category]
 		all_categories = other_categories + [category + ' freq']
 		return all_categories, other_categories
@@ -384,7 +385,7 @@ class ScatterChart:
 	                                                           df,
 	                                                           other_categories,
 	                                                           transform):
-		not_counts = df[[c + ' freq'  for c in other_categories]].sum(axis=1)
+		not_counts = df[[c + ' freq' for c in other_categories]].sum(axis=1)
 		counts = df[category + ' freq']
 		x_data_raw = transform(not_counts, df.index, counts)
 		y_data_raw = transform(counts, df.index, not_counts)
@@ -402,10 +403,7 @@ class ScatterChart:
 		return vec + np.random.rand(1, len(vec))[0] * self.scatterchartdata.jitter
 
 	def _term_rank_score_and_frequency_df(self, all_categories, category, other_categories, scores):
-		term_ranker = self.scatterchartdata.term_ranker(self.term_doc_matrix)
-		if self.scatterchartdata.use_non_text_features:
-			term_ranker.use_non_text_features()
-		df = term_ranker.get_ranks()
+		df = self._get_term_category_frequencies()
 		if self.x_coords is not None:
 			df['x'] = self.x_coords
 			df['y'] = self.y_coords
@@ -425,7 +423,7 @@ class ScatterChart:
 		if scores is None:
 			scores = self._get_default_scores(category, other_categories, df)
 		# np.array(self.term_doc_matrix.get_rudder_scores(category))
-		# df['category score'] = np.array(self.term_doc_matrix.get_rudder_scores(category))
+		# convention_df['category score'] = np.array(self.term_doc_matrix.get_rudder_scores(category))
 		category_column_name = category + ' freq'
 		df['category score'] = CornerScore.get_scores_for_category(
 			df[category_column_name],
@@ -489,7 +487,6 @@ class ScatterChart:
 	def _term_importance_ranks(self, category, df):
 		return np.array([df['category score rank'], df['not category score rank']]).min(axis=0)
 
-
 	def draw(self,
 	         category,
 	         num_top_words_to_annotate=4,
