@@ -1,6 +1,6 @@
 import numpy as np
 
-from scattertext import ScatterChart
+from scattertext import ScatterChart, TermCategoryFrequencies
 from scattertext import percentile_alphabetical
 from scattertext.Corpus import Corpus
 from scattertext.DocsAndLabelsFromCorpus import DocsAndLabelsFromCorpus, DocsAndLabelsFromCorpusSample
@@ -13,7 +13,7 @@ class ScatterChartExplorer(ScatterChart):
 		'''See ScatterChart.  This lets you click on terms to see what contexts they tend to appear in.
 
 		'''
-		assert isinstance(corpus, Corpus)
+		assert (isinstance(corpus, Corpus)) or (isinstance(corpus, TermCategoryFrequencies))
 		ScatterChart.__init__(self, corpus, **kwargs)
 
 	def to_dict(self,
@@ -30,7 +30,8 @@ class ScatterChartExplorer(ScatterChart):
 	            neutral_categories=None,
 	            extra_categories=None,
 	            neutral_category_name=None,
-	            extra_category_name=None):
+	            extra_category_name=None,
+	            background_scorer=None):
 		'''
 
 		Parameters
@@ -66,10 +67,12 @@ class ScatterChartExplorer(ScatterChart):
 		extra_category_name : str
 			"Extra" by default. Only active if show_neutral and show_extra are true. Name of the
 			extra column.
+		background_scorer : CharacteristicScorer, optional
+			Used for bg scores
 
 		Returns
 		-------
-		dictionary {info: {category_name: ..., not_category_name},
+		dictionary {info: {'category_name': full category name, ...},
 												 docs: {'texts': [doc1text, ...],
 												        'labels': [1, 0, ...],
 												        'meta': ['<b>blah</b>', '<b>blah</b>']}
@@ -93,8 +96,8 @@ class ScatterChartExplorer(ScatterChart):
 		                         title_case_names=title_case_names,
 		                         not_categories=not_categories,
 		                         neutral_categories=neutral_categories,
-		                         extra_categories=extra_categories
-		                         )
+		                         extra_categories=extra_categories,
+		                         background_scorer=background_scorer)
 		docs_getter = self._make_docs_getter(max_docs_per_category, alternative_text_field)
 		if neutral_category_name is None:
 			neutral_category_name = 'Neutral'
