@@ -1,7 +1,6 @@
-import pkgutil
+import re
 import re
 import warnings
-from io import StringIO
 
 import numpy as np
 import pandas as pd
@@ -18,6 +17,8 @@ from scattertext.CSRMatrixTools import delete_columns
 from scattertext.Common import DEFAULT_BETA, DEFAULT_SCALER_ALGO, DEFAULT_BACKGROUND_SCALER_ALGO, \
 	DEFAULT_BACKGROUND_BETA
 from scattertext.FeatureOuput import FeatureLister
+from scattertext.frequencyreaders.DefaultBackgroundFrequencies import DefaultBackgroundFrequencies
+from scattertext.indexstore import IndexStore, IndexStoreFromList
 from scattertext.termranking import AbsoluteFrequencyRanker
 from scattertext.termscoring.CornerScore import CornerScore
 
@@ -30,7 +31,9 @@ SPACY_ENTITY_TAGS = ['person', 'norp', 'facility', 'org', 'gpe',
                      'type', 'date', 'time', 'percent', 'money', 'quantity',
                      'ordinal', 'cardinal']
 
-MY_ENGLISH_STOP_WORDS = set(ENGLISH_STOP_WORDS) |  {'hasn', 'won', 'don', 'haven', 'shouldn', 'isn', 'couldn', 'wouldn', 'aren', 'didn', 'wasn', 'dosen', 'weren', 'doesn'}
+MY_ENGLISH_STOP_WORDS = set(ENGLISH_STOP_WORDS) | {'hasn', 'won', 'don', 'haven', 'shouldn', 'isn', 'couldn', 'wouldn',
+                                                   'aren', 'didn', 'wasn', 'dosen', 'weren', 'doesn'}
+
 
 class CannotCreateATermDocMatrixWithASignleCategoryException(Exception):
 	pass
@@ -91,11 +94,11 @@ class TermDocMatrix(object):
 
 		Returns
 		-------
+		TermDocMatrix
 		New Term Doc Matrix
 		'''
 		return compactor.compact(self)
 
-
 	def get_num_terms(self):
 		'''
 		Returns
@@ -137,27 +140,64 @@ class TermDocMatrix(object):
 			d[category + ' freq'] = self._X[self._y == i].sum(axis=0).A1
 		return pd.DataFrame(d).set_index('term')
 
-	def get_term_freq_df(self):
+	def get_term_freq_df(self, label_append=' freq'):
 		'''
+		Parameters
+		-------
+		label_append : str
 
 		Returns
 		-------
 		pd.DataFrame indexed on terms, with columns giving frequencies for each
+		'''
+
 		'''
 		row = self._row_category_ids()
 		newX = csr_matrix((self._X.data, (row, self._X.indices)))
 		return self._term_freq_df_from_matrix(newX)
+		'''
+		mat = self.get_term_freq_mat()
+		return pd.DataFrame(mat,
+		                    index=pd.Series(self.get_terms(), name='term'),
+		                    columns=[c + label_append for c in self.get_categories()])
 
-	def get_term_doc_count_df(self):
+	def get_term_freq_mat(self):
+		'''
+		Returns
+		-------
+		np.array with columns as categories and rows as terms
+		'''
+		freq_mat = np.zeros(shape=(len(self.get_terms()), len(self.get_categories())), dtype=int)
+		for cat_i in range(len(self._category_idx_store._i2val)):
+			freq_mat[:, cat_i] = self._X[self._y == cat_i, :].sum(axis=0)
+		return freq_mat
+
+	def get_term_count_mat(self):
+		'''
+		Returns
+		-------
+		np.array with columns as categories and rows as terms
+		'''
+		freq_mat = np.zeros(shape=(len(self.get_terms()), len(self.get_categories())), dtype=int)
+		for cat_i in range(len(self._category_idx_store._i2val)):
+			X = (self._X[self._y == cat_i, :] > 0).astype(int)
+			freq_mat[:, cat_i] = X.sum(axis=0)
+		return freq_mat
+
+	def get_term_doc_count_df(self, label_append=' freq'):
 		'''
 
 		Returns
 		-------
 		pd.DataFrame indexed on terms, with columns the number of documents each term appeared in
 		'''
-		row = self._row_category_ids()
-		newX = csr_matrix(((self._X.data > 0).astype(int), (row, self._X.indices)))
-		return self._term_freq_df_from_matrix(newX)
+		# row = self._row_category_ids()
+		# newX = csr_matrix(((self._X.data > 0).astype(int), (row, self._X.indices)))
+		# return self._term_freq_df_from_matrix(newX)
+		mat = self.get_term_count_mat()
+		return pd.DataFrame(mat,
+		                    index=self.get_terms(),
+		                    columns=[c + label_append for c in self.get_categories()])
 
 	def _term_freq_df_from_matrix(self, catX):
 		return self._get_freq_df_using_idx_store(catX, self._term_idx_store)
@@ -221,11 +261,14 @@ class TermDocMatrix(object):
 		-------
 		A new TermDocumentMatrix consisting of only unigrams in the current TermDocumentMatrix.
 		'''
-		terms_to_ignore = [term for term
-		                   in self._term_idx_store._i2val
-		                   if ' ' in term or "'" in term]
+		terms_to_ignore = self._get_non_unigrams()
 		return self.remove_terms(terms_to_ignore)
 
+	def _get_non_unigrams(self):
+		return [term for term
+		        in self._term_idx_store._i2val
+		        if ' ' in term or "'" in term]
+
 	def remove_infrequent_words(self, minimum_term_count, term_ranker=AbsoluteFrequencyRanker):
 		'''
 		Returns
@@ -332,6 +375,10 @@ class TermDocMatrix(object):
 		-------
 		TermDocMatrix, new object with terms removed.
 		'''
+		idx_to_delete_list = self._build_term_index_list(ignore_absences, terms)
+		return self.remove_terms_by_indices(idx_to_delete_list)
+
+	def _build_term_index_list(self, ignore_absences, terms):
 		idx_to_delete_list = []
 		for term in terms:
 			if term not in self._term_idx_store:
@@ -339,7 +386,7 @@ class TermDocMatrix(object):
 					raise KeyError('Term %s not found' % (term))
 				continue
 			idx_to_delete_list.append(self._term_idx_store.getidx(term))
-		return self.remove_terms_by_indices(idx_to_delete_list)
+		return idx_to_delete_list
 
 	def keep_only_these_categories(self, categories, ignore_absences=False):
 		'''
@@ -396,7 +443,8 @@ class TermDocMatrix(object):
 		                  for i in range(intermediate_y.max() + 1)])[intermediate_y]
 
 		new_metadata_idx_store = self._metadata_idx_store
-		if len(self._metadata_idx_store):
+
+		if self.metadata_in_use():
 			meta_idx_to_delete = np.nonzero(new_mX.sum(axis=0).A1 == 0)[0]
 			new_metadata_idx_store = self._metadata_idx_store.batch_delete_idx(meta_idx_to_delete)
 
@@ -404,11 +452,25 @@ class TermDocMatrix(object):
 		new_term_idx_store = self._term_idx_store.batch_delete_idx(term_idx_to_delete)
 		new_X = delete_columns(new_X, term_idx_to_delete)
 
-		term_doc_mat_to_ret = self._make_new_term_doc_matrix(new_X, new_mX, new_y, new_term_idx_store,
+		term_doc_mat_to_ret = self._make_new_term_doc_matrix(new_X,
+		                                                     new_mX,
+		                                                     new_y,
+		                                                     new_term_idx_store,
 		                                                     new_category_idx_store,
-		                                                     new_metadata_idx_store, ~np.isin(self._y, idx_to_delete_list))
+		                                                     new_metadata_idx_store,
+		                                                     ~np.isin(self._y, idx_to_delete_list))
 		return term_doc_mat_to_ret
 
+	def metadata_in_use(self):
+		'''
+		Returns True if metadata values are in term doc matrix.
+
+		Returns
+		-------
+		bool
+		'''
+		return len(self._metadata_idx_store) > 0
+
 	def remove_terms_by_indices(self, idx_to_delete_list):
 		'''
 		Parameters
@@ -419,8 +481,7 @@ class TermDocMatrix(object):
 		-------
 		TermDocMatrix
 		'''
-		new_term_idx_store = self._term_idx_store.batch_delete_idx(idx_to_delete_list)
-		new_X = delete_columns(self._X, idx_to_delete_list)
+		new_X, new_term_idx_store = self._get_X_after_delete_terms(idx_to_delete_list)
 		return self._make_new_term_doc_matrix(new_X,
 		                                      self._mX,
 		                                      self._y,
@@ -429,6 +490,11 @@ class TermDocMatrix(object):
 		                                      self._metadata_idx_store,
 		                                      self._y == self._y)
 
+	def _get_X_after_delete_terms(self, idx_to_delete_list):
+		new_term_idx_store = self._term_idx_store.batch_delete_idx(idx_to_delete_list)
+		new_X = delete_columns(self._X, idx_to_delete_list)
+		return new_X, new_term_idx_store
+
 	def remove_terms_used_in_less_than_num_docs(self, threshold):
 		'''
 		Parameters
@@ -615,6 +681,18 @@ class TermDocMatrix(object):
 	def _get_continuous_version_boolean_y(self, y_bool):
 		return 1000 * (y_bool * 2. - 1)
 
+	def get_doc_lengths(self):
+		'''
+		Returns a list of document lengths in words
+
+		Returns
+		-------
+		np.array
+		'''
+		idx_to_delete_list = self._build_term_index_list(True, self._get_non_unigrams())
+		unigram_X, _ = self._get_X_after_delete_terms(idx_to_delete_list)
+		return unigram_X.sum(axis=1).A1
+
 	def get_scaled_f_scores(self,
 	                        category,
 	                        scaler_algo=DEFAULT_SCALER_ALGO,
@@ -782,7 +860,7 @@ class TermDocMatrix(object):
 	def get_background_corpus(self):
 		if self._background_corpus is not None:
 			return self._background_corpus
-		return None
+		return DefaultBackgroundFrequencies.get_background_frequency_df(self._unigram_frequency_path)
 
 	def get_term_and_background_counts(self):
 		'''
@@ -817,17 +895,7 @@ class TermDocMatrix(object):
 	def _get_background_unigram_frequencies(self):
 		if self.get_background_corpus() is not None:
 			return self.get_background_corpus()
-		if self._unigram_frequency_path:
-			unigram_freq_table_buf = open(self._unigram_frequency_path)
-		else:
-			unigram_freq_table_buf = StringIO(pkgutil.get_data('scattertext', 'data/count_1w.txt')
-			                                  .decode('utf-8'))
-		to_ret = (pd.read_table(unigram_freq_table_buf,
-		                        names=['word', 'background'])
-		          .sort_values(ascending=False, by='background')
-		          .drop_duplicates(['word'])
-		          .set_index('word'))
-		return to_ret
+		return DefaultBackgroundFrequencies.get_background_frequency_df(self._unigram_frequency_path)
 
 	def list_extra_features(self):
 		'''
@@ -882,3 +950,87 @@ class TermDocMatrix(object):
 
 	def _get_percentiles_from_freqs(self, freqs):
 		return rankdata(freqs) / len(freqs)
+
+	def get_term_category_frequencies(self, scatterchartdata):
+		'''
+		Applies the ranker in scatterchartdata to term-category frequencies.
+
+		Parameters
+		----------
+		scatterchartdata : ScatterChartData
+
+		Returns
+		-------
+		pd.DataFrame
+		'''
+		term_ranker = scatterchartdata.term_ranker(self)
+		if scatterchartdata.use_non_text_features:
+			term_ranker.use_non_text_features()
+		return term_ranker.get_ranks()
+
+	def apply_ranker(self, term_ranker):
+		'''
+		Parameters
+		----------
+		term_ranker : TermRanker
+
+		Returns
+		-------
+		pd.Dataframe
+		'''
+		return term_ranker(self).get_ranks()
+
+	def get_category_ids(self):
+		'''
+		Returns array of category ids
+
+		Returns
+		-------
+		np.array
+		'''
+		return self._y
+
+	def get_term_doc_mat(self):
+		'''
+		Returns sparse matrix representation of term-doc-matrix
+
+		Returns
+		-------
+		scipy.sparse.csr_matrix
+		'''
+		return self._X
+
+	def get_category_index_store(self):
+		'''
+		Returns IndexStore object mapping categories to ids
+
+		Returns
+		-------
+		IndexStore
+		'''
+		return self._category_idx_store
+
+	def recategorize(self, new_categories):
+		'''
+		Parameters
+		----------
+		new_categories : array like
+		String names of new categories. Length should be equal to number of documents
+
+		Returns
+		-------
+		TermDocMatrix
+		'''
+		assert len(new_categories) == self.get_num_docs()
+
+		new_category_idx_store = IndexStoreFromList.build(set(new_categories))
+		new_y = np.array(new_category_idx_store.getidxstrictbatch(new_categories))
+
+		new_tdm = self._make_new_term_doc_matrix(self._X,
+		                                         self._mX,
+		                                         new_y,
+		                                         self._term_idx_store,
+		                                         new_category_idx_store,
+		                                         self._metadata_idx_store,
+		                                         new_y == new_y)
+		return new_tdm
