@@ -1,14 +1,21 @@
 from __future__ import print_function
 
-import re
-
-version = [0, 0, 2, 20]
+version = [0, 0, 2, 28]
 __version__ = '.'.join([str(e) for e in version])
 
+import re
 import warnings
 import numpy as np
 import pandas as pd
 import scattertext.viz
+
+from scattertext.characteristic.DenseRankCharacteristicness import DenseRankCharacteristicness
+from scattertext.CorpusDF import CorpusDF
+from scattertext.CorpusFromFeatureDict import CorpusFromFeatureDict
+from scattertext.TermCategoryFrequencies import TermCategoryFrequencies
+from scattertext.diachronic import GanttChart
+from scattertext.features.FeatsFromTopicModel import FeatsFromTopicModel
+from scattertext.termscoring.BM25Difference import BM25Difference
 from scattertext import SampleCorpora
 from scattertext import Scalers, ScatterChart
 from scattertext import termranking
@@ -33,15 +40,18 @@ from scattertext.TermDocMatrixFactory import TermDocMatrixFactory, FeatsFromDoc
 from scattertext.TermDocMatrixFilter import TermDocMatrixFilter, filter_bigrams_by_pmis
 from scattertext.TermDocMatrixFromPandas import TermDocMatrixFromPandas
 from scattertext.TermDocMatrixFromScikit import TermDocMatrixFromScikit
-from scattertext.WhitespaceNLP import whitespace_nlp, whitespace_nlp_with_sentences, tweet_tokenzier_factory
+from scattertext.WhitespaceNLP import whitespace_nlp, whitespace_nlp_with_sentences, \
+	tweet_tokenzier_factory
 from scattertext.external.phrasemachine import phrasemachine
 from scattertext.features.FeatsFromGeneralInquirer import FeatsFromGeneralInquirer
 from scattertext.features.FeatsFromOnlyEmpath import FeatsFromOnlyEmpath
 from scattertext.features.FeatsFromSpacyDoc import FeatsFromSpacyDoc
+from scattertext.features.UnigramsFromSpacyDoc import UnigramsFromSpacyDoc
 from scattertext.features.FeatsFromSpacyDocAndEmpath import FeatsFromSpacyDocAndEmpath
 from scattertext.features.FeatsFromSpacyDocOnlyEmoji import FeatsFromSpacyDocOnlyEmoji
 from scattertext.features.FeatsFromSpacyDocOnlyNounChunks import FeatsFromSpacyDocOnlyNounChunks
-from scattertext.features.PhraseMachinePhrases import PhraseMachinePhrases, PhraseMachinePhrasesAndUnigrams
+from scattertext.features.PhraseMachinePhrases \
+	import PhraseMachinePhrases, PhraseMachinePhrasesAndUnigrams
 from scattertext.indexstore import IndexStoreFromDict
 from scattertext.indexstore import IndexStoreFromList
 from scattertext.indexstore.IndexStore import IndexStore
@@ -51,11 +61,13 @@ from scattertext.semioticsquare import SemioticSquare, FourSquareAxis
 from scattertext.semioticsquare.FourSquare import FourSquare
 from scattertext.termranking import OncePerDocFrequencyRanker
 from scattertext.termscoring.RankDifference import RankDifference
-from scattertext.termscoring.ScaledFScore import InvalidScalerException, ScaledFScorePresets, ScaledFZScore, \
-	ScaledFZScorePrior, ScaledFScorePresetsNeg1To1
+from scattertext.termscoring.ScaledFScore import InvalidScalerException, ScaledFScorePresets, \
+	ScaledFZScore, ScaledFZScorePrior, ScaledFScorePresetsNeg1To1
 from scattertext.termsignificance.LogOddsRatioAddOne import LogOddsRatioAddOne
-from scattertext.termsignificance.LogOddsRatioInformativeDirichletPiror import LogOddsRatioInformativeDirichletPrior
-from scattertext.termsignificance.LogOddsRatioUninformativeDirichletPrior import LogOddsRatioUninformativeDirichletPrior
+from scattertext.termsignificance.LogOddsRatioInformativeDirichletPiror \
+	import LogOddsRatioInformativeDirichletPrior
+from scattertext.termsignificance.LogOddsRatioUninformativeDirichletPrior \
+	import LogOddsRatioUninformativeDirichletPrior
 from scattertext.termsignificance.ScaledFScoreSignificance import ScaledFScoreSignificance
 from scattertext.termsignificance.TermSignificance import TermSignificance
 from scattertext.viz import VizDataAdapter, HTMLVisualizationAssembly
@@ -64,6 +76,11 @@ from scattertext.semioticsquare.FourSquareAxis import FourSquareAxes
 from scattertext.termcompaction.ClassPercentageCompactor import ClassPercentageCompactor
 from scattertext.termcompaction.CompactTerms import CompactTerms
 from scattertext.termcompaction.PhraseSelector import PhraseSelector
+from scattertext.topicmodel.SentencesForTopicModeling import SentencesForTopicModeling
+from scattertext.frequencyreaders.DefaultBackgroundFrequencies import DefaultBackgroundFrequencies
+from scattertext.termcompaction.DomainCompactor import DomainCompactor
+from scattertext.termscoring.ZScores import ZScores
+from scattertext.termscoring.RelativeEntropy import RelativeEntropy
 
 
 def produce_scattertext_html(term_doc_matrix,
@@ -150,7 +167,7 @@ def produce_scattertext_explorer(corpus,
                                  rescale_x=None,
                                  rescale_y=None,
                                  singleScoreMode=False,
-                                 sort_by_dist=True,
+                                 sort_by_dist=False,
                                  reverse_sort_scores_for_not_category=True,
                                  use_full_doc=False,
                                  transform=percentile_alphabetical,
@@ -189,7 +206,14 @@ def produce_scattertext_explorer(corpus,
                                  show_extra=False,
                                  extra_category_name=None,
                                  censor_points=True,
-                                 center_label_over_points=False):
+                                 center_label_over_points=False,
+                                 x_axis_labels=None,
+                                 y_axis_labels=None,
+                                 topic_model_term_lists=None,
+                                 topic_model_preview_size=10,
+                                 vertical_lines=None,
+                                 characteristic_scorer=None,
+                                 return_data=False):
 	'''Returns html code of visualization.
 
 	Parameters
@@ -346,6 +370,24 @@ def produce_scattertext_explorer(corpus,
 	center_label_over_points : bool, default False
 		Center a label over points, or try to find a position near a point that
 		doesn't overlap anything else.
+	x_axis_labels: list, default None
+		List of string value-labels to show at evenly spaced intervals on the x-axis.
+		Low, medium, high are defaults.
+	y_axis_labels : list, default None
+		List of string value-labels to show at evenly spaced intervals on the y-axis.
+		Low, medium, high are defaults.
+	topic_model_term_lists : dict default None
+		Dict of metadata name (str) -> List of string terms in metatdata. These will be bolded
+		in query in context results.
+	topic_model_preview_size : int default 10
+		Number of terms in topic model to show as a preview.
+	vertical_lines : list default None
+		List of floats corresponding to points on the x-axis to draw vertical lines
+	characteristic_scorer : CharacteristicScorer default None
+		Used for bg scores
+	return_data : bool default False
+		Return a dict containing the output of `ScatterChartExplorer.to_dict` instead of
+		an html.
 
 	Returns
 	-------
@@ -363,20 +405,14 @@ def produce_scattertext_explorer(corpus,
 	if term_ranker is None:
 		term_ranker = termranking.AbsoluteFrequencyRanker
 
-	if category_name is None:
-		category_name = category
-
-	if not_category_name is None:
-		if not_categories is not None and len(not_categories) == 1:
-			not_category_name = not_categories[0]
-		else:
-			not_category_name = ('Not' if category_name[0].isupper() else 'not') + ' ' + category_name
+	category_name, not_category_name = get_category_names(category, category_name, not_categories, not_category_name)
 
 	if not_categories is None:
 		not_categories = [c for c in corpus.get_categories() if c != category]
 
 	if term_scorer:
-		tdf = term_ranker(corpus).get_ranks()
+		# tdf = term_ranker(corpus).get_ranks()
+		tdf = corpus.apply_ranker(term_ranker)
 		cat_freqs = tdf[category + ' freq']
 		if not_categories:
 			not_cat_freqs = tdf[[c + ' freq' for c in not_categories]].sum(axis=1)
@@ -390,6 +426,9 @@ def produce_scattertext_explorer(corpus,
 			"The argument name 'pmi_filter_thresold' has been deprecated. Use 'pmi_threshold_coefficient' in its place",
 			DeprecationWarning)
 
+	if use_non_text_features:
+		pmi_threshold_coefficient = 0
+
 	scatter_chart_explorer = ScatterChartExplorer(corpus,
 	                                              minimum_term_frequency=minimum_term_frequency,
 	                                              minimum_not_category_term_frequency=minimum_not_category_term_frequency,
@@ -400,7 +439,7 @@ def produce_scattertext_explorer(corpus,
 	                                              term_ranker=term_ranker,
 	                                              use_non_text_features=use_non_text_features,
 	                                              term_significance=term_significance,
-	                                              terms_to_include=terms_to_include)
+	                                              terms_to_include=terms_to_include, )
 	if ((x_coords is None and y_coords is not None)
 			or (y_coords is None and x_coords is not None)):
 		raise Exception("Both x_coords and y_coords need to be passed or both left blank")
@@ -411,6 +450,8 @@ def produce_scattertext_explorer(corpus,
 		                                          rescale_y=rescale_y,
 		                                          original_x=original_x,
 		                                          original_y=original_y)
+	if topic_model_term_lists is not None:
+		scatter_chart_explorer.inject_metadata_term_lists(topic_model_term_lists)
 	html_base = None
 	if semiotic_square:
 		html_base = get_semiotic_square_html(num_terms_semiotic_square,
@@ -427,7 +468,10 @@ def produce_scattertext_explorer(corpus,
 	                                                    neutral_category_name=neutral_category_name,
 	                                                    extra_category_name=extra_category_name,
 	                                                    neutral_categories=neutral_categories,
-	                                                    extra_categories=extra_categories)
+	                                                    extra_categories=extra_categories,
+	                                                    background_scorer=characteristic_scorer)
+	if return_data:
+		return scatter_chart_data
 	return HTMLVisualizationAssembly(VizDataAdapter(scatter_chart_data),
 	                                 width_in_pixels=width_in_pixels,
 	                                 height_in_pixels=height_in_pixels,
@@ -455,13 +499,28 @@ def produce_scattertext_explorer(corpus,
 	                                 show_axes=show_axes,
 	                                 show_extra=show_extra,
 	                                 do_censor_points=censor_points,
-	                                 center_label_over_points=center_label_over_points) \
+	                                 center_label_over_points=center_label_over_points,
+	                                 x_axis_labels=x_axis_labels,
+	                                 y_axis_labels=y_axis_labels,
+	                                 topic_model_preview_size=topic_model_preview_size,
+	                                 vertical_lines=vertical_lines) \
 		.to_html(protocol=protocol,
 	           d3_url=d3_url,
 	           d3_scale_chromatic_url=d3_scale_chromatic_url,
 	           html_base=html_base)
 
 
+def get_category_names(category, category_name, not_categories, not_category_name):
+	if category_name is None:
+		category_name = category
+	if not_category_name is None:
+		if not_categories is not None and len(not_categories) == 1:
+			not_category_name = not_categories[0]
+		else:
+			not_category_name = ('Not' if category_name[0].isupper() else 'not') + ' ' + category_name
+	return category_name, not_category_name
+
+
 def get_semiotic_square_html(num_terms_semiotic_square, semiotic_square):
 	semiotic_square_html = None
 	if semiotic_square:
@@ -1121,6 +1180,82 @@ def produce_projection_explorer(corpus,
 	return html
 
 
+def produce_characteristic_explorer(corpus,
+                                    category,
+                                    category_name=None,
+                                    not_category_name=None,
+                                    not_categories=None,
+                                    characteristic_scorer=DenseRankCharacteristicness(),
+                                    term_ranker=termranking.AbsoluteFrequencyRanker,
+                                    term_scorer=RankDifference(),
+                                    **kwargs):
+	'''
+	Parameters
+	----------
+	corpus : Corpus
+		It is highly recommended to use a stoplisted, unigram corpus-- `corpus.get_stoplisted_unigram_corpus()`
+	category : str
+	category_name : str
+	not_category_name : str
+	not_categories : list
+	characteristic_scorer : CharacteristicScorer
+	term_ranker
+	term_scorer
+	term_acceptance_re : SRE_Pattern
+		Regular expression to identify valid terms
+	kwargs : dict
+		remaining produce_scattertext_explorer keywords
+
+	Returns
+	-------
+	str HTML of visualization
+
+	'''
+	if not_categories is None:
+		not_categories = [c for c in corpus.get_categories() if c != category]
+
+	category_name, not_category_name = get_category_names(
+		category, category_name, not_categories, not_category_name)
+
+	zero_point, characteristic_scores = characteristic_scorer.get_scores(corpus)
+	corpus = corpus.remove_terms(set(corpus.get_terms()) - set(characteristic_scores.index))
+	characteristic_scores = characteristic_scores.loc[corpus.get_terms()]
+	term_freq_df = term_ranker(corpus).get_ranks()
+	scores = term_scorer.get_scores(
+		term_freq_df[category + ' freq'],
+		term_freq_df[[c + ' freq' for c in not_categories]].sum(axis=1)
+	)
+	kwargs['scores'] = kwargs.get('scores', scores)
+	max_score = np.floor(np.max(kwargs['scores']) * 100) / 100
+	min_score = np.ceil(np.min(kwargs['scores']) * 100) / 100
+	if min_score < 0 and max_score > 0:
+		central = 0
+	else:
+		central = 0.5
+	scores_scaled_for_charting = scale_neg_1_to_1_with_zero_mean_abs_max(kwargs['scores'])
+	html = produce_scattertext_explorer(
+		corpus=corpus,
+		category=category,
+		category_name=category_name,
+		not_category_name=not_category_name,
+		not_categories=not_categories,
+		minimum_term_frequency=0,
+		sort_by_dist=False,
+		x_coords=characteristic_scores,
+		y_coords=scores_scaled_for_charting,
+		y_axis_labels=kwargs.get('y_axis_labels',
+		                         ['More ' + not_category_name,
+		                          'Even',
+		                          'More ' + category_name]),
+		x_label=kwargs.get('x_label', 'Characteristic to Corpus'),
+		y_label=kwargs.get('y_label', term_scorer.get_name()),
+		vertical_lines=kwargs.get('vertical_lines', []),
+		characteristic_scorer=kwargs.get('characteristic_scorer', characteristic_scorer),
+		**kwargs
+	)
+	return html
+
+
 def sparse_explorer(corpus,
                     category,
                     scores,
