@@ -1,7 +1,6 @@
 import itertools
 from collections import Counter
 
-from scattertext.features import FeatsFromSpacyDoc
 from scattertext.ParsedCorpus import ParsedCorpus
 
 
@@ -95,6 +94,25 @@ class CorpusAdapterForGensim(object):
 		                         for doc in corpus.get_parsed_docs()])
 
 
+class CorpusSentenceIterator(object):
+	@staticmethod
+	def get_sentences(corpus):
+		'''
+		Parameters
+		----------
+		corpus, ParsedCorpus
+
+		Returns
+		-------
+		iter: [sentence1word1, ...], [sentence2word1, ...]
+		'''
+		assert isinstance(corpus, ParsedCorpus)
+		return itertools.chain(*[[[corpus._term_idx_store.getidxstrict(t.lower_) for t in sent
+		                           if not t.is_punct]
+		                          for sent in doc.sents]
+		                         for doc in corpus.get_parsed_docs()])
+
+
 class Word2VecFromParsedCorpus(object):
 	def __init__(self, corpus, word2vec_model=None):
 		'''
@@ -169,7 +187,10 @@ class Word2VecFromParsedCorpus(object):
 		                         sorted_vocab=1)
 
 	def _scan_and_build_vocab(self):
-		self.model.scan_vocab(CorpusAdapterForGensim.get_sentences(self.corpus))
+		try:
+			self.model.scan_vocab(CorpusAdapterForGensim.get_sentences(self.corpus))
+		except:
+			pass
 		self.model.build_vocab(CorpusAdapterForGensim.get_sentences(self.corpus))
 
 
@@ -177,5 +198,8 @@ class Word2VecFromParsedCorpusBigrams(Word2VecFromParsedCorpus):
 	def _scan_and_build_vocab(self):
 		from gensim.models import Phrases
 		bigram_transformer = Phrases(CorpusAdapterForGensim.get_sentences(self.corpus))
-		self.model.scan_vocab(bigram_transformer[CorpusAdapterForGensim.get_sentences(self.corpus)])
+		try:
+			self.model.scan_vocab(CorpusAdapterForGensim.get_sentences(self.corpus))
+		except:
+			pass
 		self.model.build_vocab(bigram_transformer[CorpusAdapterForGensim.get_sentences(self.corpus)])
