@@ -4,6 +4,7 @@ from unittest import TestCase
 
 import numpy as np
 import pandas as pd
+from scattertext.indexstore import IndexStore
 from sklearn.feature_extraction.stop_words import ENGLISH_STOP_WORDS
 from sklearn.linear_model import LinearRegression
 
@@ -16,6 +17,7 @@ from scattertext.WhitespaceNLP import whitespace_nlp
 from scattertext.termscoring.ScaledFScore import InvalidScalerException
 from scattertext.test.test_corpusFromPandas import get_docs_categories
 from scattertext.test.test_semioticSquare import get_docs_categories_semiotic
+from scattertext.test.test_termDocMatrixFactory import build_hamlet_jz_corpus_with_meta
 
 
 def make_a_test_term_doc_matrix():
@@ -189,7 +191,7 @@ class TestTermDocMat(TestCase):
 		                 ['another', 'blah', 'blah blah'])
 
 		# to do: come up with faster way of testing fisher
-		# self.assertEqual(list(df.sort_values(by='fisher pval', ascending=True).index[:3]),
+		# self.assertEqual(list(convention_df.sort_values(by='fisher pval', ascending=True).index[:3]),
 		#                 ['another', 'blah', 'blah blah'])
 
 		self.assertEqual(list(df.sort_values(by='rudder', ascending=True).index[:3]),
@@ -246,8 +248,8 @@ class TestTermDocMat(TestCase):
 		self.assertEqual(swiftless_constructed._X.shape, swiftless._X.shape)
 		self.assertEqual(set(swiftless_constructed.get_terms()),
 		                 set(swiftless.get_terms()))
-		np.testing.assert_equal(swiftless_constructed.get_texts(),
-		                        swiftless.get_texts())
+		pd.testing.assert_series_equal(swiftless_constructed.get_texts(),
+		                               swiftless.get_texts())
 
 	def test_get_category_names_by_row(self):
 		hamlet = get_hamlet_term_doc_matrix()
@@ -278,8 +280,8 @@ class TestTermDocMat(TestCase):
 
 	def test_get_background_corpus(self):
 		tdm = get_hamlet_term_doc_matrix()
-		background = tdm.get_background_corpus()
-		self.assertEqual(background, None)
+		#background = tdm.get_background_corpus()
+		#self.assertEqual(background, None)
 		back_df = pd.DataFrame({'word': ['a', 'bee'], 'background': [3, 1]})
 		tdm.set_background_corpus(back_df)
 		self.assertEqual(tdm.get_background_corpus().to_dict(),
@@ -289,8 +291,8 @@ class TestTermDocMat(TestCase):
 		                 set(['word', 'background']))
 
 	# to do: come up with faster way of testing fisher
-	# df = hamlet.get_fisher_scores_vs_background()
-	# self.assertEqual(list(df.sort_values(by='Bonferroni-corrected p-values', ascending=True).index[:3]),
+	# convention_df = hamlet.get_fisher_scores_vs_background()
+	# self.assertEqual(list(convention_df.sort_values(by='Bonferroni-corrected p-values', ascending=True).index[:3]),
 	#                 ['voltimand', 'knavish', 'mobled'])
 
 	def test_log_reg(self):
@@ -305,8 +307,41 @@ class TestTermDocMat(TestCase):
 		self.assertEqual(list(df.sort_values(by='logreg', ascending=False).index[:3]),
 		                 ['the', 'starts', 'incorporal'])
 
+	def test_get_doc_lengths(self):
+		hamlet = get_hamlet_term_doc_matrix()
+		hamlet_unigram = hamlet.get_unigram_corpus()
+		self.assertEqual(len(hamlet.get_doc_lengths()), hamlet.get_num_docs())
+		np.testing.assert_array_equal(hamlet.get_doc_lengths(), hamlet_unigram._X.sum(axis=1).A1)
 
-# ['hamlet', 'hamlet,', 'the'])
+	def test_get_category_ids(self):
+		hamlet = get_hamlet_term_doc_matrix()
+		np.testing.assert_array_equal(hamlet.get_category_ids(), hamlet._y)
+
+	def test_metadata_in_use(self):
+		hamlet = get_hamlet_term_doc_matrix()
+		self.assertFalse(hamlet.metadata_in_use())
+		hamlet_meta = build_hamlet_jz_corpus_with_meta()
+		self.assertTrue(hamlet_meta.metadata_in_use())
+
+	def test_get_term_doc_mat(self):
+		hamlet = get_hamlet_term_doc_matrix()
+		X = hamlet.get_term_doc_mat()
+		np.testing.assert_array_equal(X.shape, (hamlet.get_num_docs(), hamlet.get_num_terms()))
+
+	def test_get_category_index_store(self):
+		hamlet = get_hamlet_term_doc_matrix()
+		idxstore = hamlet.get_category_index_store()
+		self.assertIsInstance(idxstore, IndexStore)
+		self.assertEquals(idxstore.getvals(), {'hamlet', 'not hamlet'})
+
+	def test_recategorize(self):
+		hamlet = get_hamlet_term_doc_matrix()
+		newcats = ['cat' + str(i % 3) for i in range(hamlet.get_num_docs())]
+		newhamlet = hamlet.recategorize(newcats)
+		self.assertEquals(set(newhamlet.get_term_freq_df('').columns),
+		                  {'cat0', 'cat1', 'cat2'})
+		self.assertEquals(set(hamlet.get_term_freq_df('').columns),
+		                  {'hamlet', 'not hamlet'})
 
 
 def get_hamlet_term_doc_matrix():
