@@ -38,8 +38,11 @@ class HTMLVisualizationAssembly(object):
 	             show_axes=True,
 	             show_extra=False,
 	             do_censor_points=True,
-	             center_label_over_points=False
-	             ):
+	             center_label_over_points=False,
+	             x_axis_labels=None,
+	             y_axis_labels=None,
+	             topic_model_preview_size=10,
+	             vertical_lines = None):
 		'''
 
 		Parameters
@@ -51,7 +54,7 @@ class HTMLVisualizationAssembly(object):
 		height_in_pixels : int, optional
 			height of viz in pixels, if None, default to 600
 		max_snippets : int, optional
-			max snippets to snow when temr is clicked, Defaults to show all
+			max snippets to snow when term is clicked, Defaults to show all
 		color : str, optional
 		  d3 color scheme
 		grey_zero_scores : bool, optional
@@ -94,9 +97,9 @@ class HTMLVisualizationAssembly(object):
 			which is a dictionary entry produced by `ScatterChartExplorer.to_dict` and
 			returns a string.
 		x_axis_values : list, default None
-			Value-labels to show on x-axis. Low, medium, high are defaults.
+			Numeric value-labels to show on x-axis which correspond to original x-values.
 		y_axis_values : list, default None
-			Value-labels to show on y-axis. Low, medium, high are defaults.
+			Numeric Value-labels to show on y-axis which correspond to original y-values.
 		color_func : str, default None
 			Javascript function to control color of a point.  Function takes a parameter
 			which is a dictionary entry produced by `ScatterChartExplorer.to_dict` and
@@ -109,6 +112,20 @@ class HTMLVisualizationAssembly(object):
 			Don't label over dots
 		center_label_over_points : bool, default False
 			Only put labels centered over point
+		x_axis_labels: list, default None
+			List of string value-labels to show at evenly spaced intervals on the x-axis.
+			Low, medium, high are defaults. This relies on d3's ticks function, which can
+			behave unpredictable. Caveat usor.
+		y_axis_labels : list, default None
+			List of string value-labels to show at evenly spaced intervals on the y-axis.
+			Low, medium, high are defaults.  This relies on d3's ticks function, which can
+			behave unpredictable. Caveat usor.
+		topic_model_preview_size : int, default 10
+			If topic models are being visualized, show the first topic_model_preview_size topics
+			in a preview.
+		vertical_lines: list, default None
+			List of scaled points along the x-axis to draw horizontal lines
+
 		'''
 		self._visualization_data = visualization_data
 		self._width_in_pixels = width_in_pixels if width_in_pixels is not None else 1000
@@ -134,11 +151,15 @@ class HTMLVisualizationAssembly(object):
 		self._get_tooltip_content = get_tooltip_content
 		self._x_axis_values = x_axis_values
 		self._y_axis_values = y_axis_values
+		self._x_axis_labels = x_axis_labels
+		self._y_axis_labels = y_axis_labels
 		self._color_func = color_func
 		self._show_axes = show_axes
 		self._show_extra = show_extra
 		self._do_censor_points = do_censor_points
 		self._center_label_over_points = center_label_over_points
+		self._topic_model_preview_size = topic_model_preview_size
+		self._vertical_lines = vertical_lines
 
 	def to_html(self,
 	            protocol='http',
@@ -238,6 +259,9 @@ class HTMLVisualizationAssembly(object):
 		def js_default_value_to_null(x):
 			return 'null' if x is None else str(x)
 
+		def js_list_or_null(x):
+			return 'null' if x is None else json.dumps(x)
+
 		def js_bool(x):
 			return 'true' if x else 'false'
 
@@ -274,5 +298,9 @@ class HTMLVisualizationAssembly(object):
 		             js_bool(self._show_axes),
 		             js_bool(self._show_extra),
 		             js_bool(self._do_censor_points),
-		             js_bool(self._center_label_over_points)]
+		             js_bool(self._center_label_over_points),
+		             js_list_or_null(self._x_axis_labels),
+		             js_list_or_null(self._y_axis_labels),
+		             js_default_value(self._topic_model_preview_size),
+		             js_list_or_null(self._vertical_lines)]
 		return 'plotInterface = buildViz(' + ','.join(arguments) + ');'
